
from diagrams import Diagram, Cluster, Edge

from diagrams.aws.management import Organizations, OrganizationsAccount, OrganizationsOrganizationalUnit
from diagrams.aws.general import Users, User
from diagrams.aws.security import IAMPermissions
with Diagram("IAM Identity Center", show=False, direction="LR"):
    gg = Users("Group")
    uu = User("User")
    pp= IAMPermissions("PermissionsSet")
    ou = OrganizationsOrganizationalUnit("PermissionsAssignments")

    with Cluster('Account: Log archive'):

        with Cluster('Group: AWSSecurityAuditPowerUsers'):

                gg_AWSSecurityAuditPowerUsers=Users("AWSSecurityAudit\nPowerUsers")
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSPowerUserAcce\nss")
                mm_AWSSecurityAuditPowerUsers=[] 
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditPowerUsers 


        with Cluster('Group: AWSControlTowerAdmins'):

                gg_AWSControlTowerAdmins=Users("AWSControlTowerA\ndmins")
                gg_AWSControlTowerAdmins \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSAdministrator\nAccess")
                mm_AWSControlTowerAdmins=[User("velez94@protonma\nil.com"),] 
                gg_AWSControlTowerAdmins \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSControlTowerAdmins 


        with Cluster('Group: AWSLogArchiveAdmins'):

                gg_AWSLogArchiveAdmins=Users("AWSLogArchiveAdm\nins")
                gg_AWSLogArchiveAdmins \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSAdministrator\nAccess")
                mm_AWSLogArchiveAdmins=[] 
                gg_AWSLogArchiveAdmins \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSLogArchiveAdmins 


        with Cluster('Group: AWSSecurityAuditors'):

                gg_AWSSecurityAuditors=Users("AWSSecurityAudit\nors")
                gg_AWSSecurityAuditors \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSReadOnlyAccess")
                mm_AWSSecurityAuditors=[] 
                gg_AWSSecurityAuditors \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditors 


        with Cluster('Group: AWSLogArchiveViewers'):

                gg_AWSLogArchiveViewers=Users("AWSLogArchiveVie\nwers")
                gg_AWSLogArchiveViewers \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSReadOnlyAccess")
                mm_AWSLogArchiveViewers=[] 
                gg_AWSLogArchiveViewers \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSLogArchiveViewers 


    with Cluster('Account: SecOps'):

        with Cluster('Group: AWSSecurityAuditPowerUsers'):

                gg_AWSSecurityAuditPowerUsers=Users("AWSSecurityAudit\nPowerUsers")
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSPowerUserAcce\nss")
                mm_AWSSecurityAuditPowerUsers=[] 
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditPowerUsers 


        with Cluster('User: w.alejovl+secops-labs@gmail.com'):

                uu_walejovlsecopslabsgmailcom=User("w.alejovl+secops\n-labs@gmail.com")
                uu_walejovlsecopslabsgmailcom \
                        - Edge(color="brown", style="dotted") \
                        - IAMPermissions("AWSAdministrator\nAccess")

        with Cluster('Group: AWSSecurityAuditors'):

                gg_AWSSecurityAuditors=Users("AWSSecurityAudit\nors")
                gg_AWSSecurityAuditors \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSReadOnlyAccess")
                mm_AWSSecurityAuditors=[] 
                gg_AWSSecurityAuditors \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditors 


    with Cluster('Account: Prod'):

        with Cluster('Group: AWSSecurityAuditPowerUsers'):

                gg_AWSSecurityAuditPowerUsers=Users("AWSSecurityAudit\nPowerUsers")
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSPowerUserAcce\nss")
                mm_AWSSecurityAuditPowerUsers=[] 
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditPowerUsers 


        with Cluster('Group: DevSecOps_Admins'):

                gg_DevSecOps_Admins=Users("DevSecOps_Admins")
                gg_DevSecOps_Admins \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSAdministrator\nAccess")
                mm_DevSecOps_Admins=[User("DevSecOpsAdm"),] 
                gg_DevSecOps_Admins \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_DevSecOps_Admins 


        with Cluster('User: w.alejovl+prod-labs@gmail.com'):

                uu_walejovlprodlabsgmailcom=User("w.alejovl+prod-l\nabs@gmail.com")
                uu_walejovlprodlabsgmailcom \
                        - Edge(color="brown", style="dotted") \
                        - IAMPermissions("AWSAdministrator\nAccess")

        with Cluster('Group: AWSSecurityAuditors'):

                gg_AWSSecurityAuditors=Users("AWSSecurityAudit\nors")
                gg_AWSSecurityAuditors \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSReadOnlyAccess")
                mm_AWSSecurityAuditors=[] 
                gg_AWSSecurityAuditors \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditors 


    with Cluster('Account: SecurityTooling'):

        with Cluster('Group: AWSSecurityAuditPowerUsers'):

                gg_AWSSecurityAuditPowerUsers=Users("AWSSecurityAudit\nPowerUsers")
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSPowerUserAcce\nss")
                mm_AWSSecurityAuditPowerUsers=[] 
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditPowerUsers 


        with Cluster('Group: SecOps_Adms'):

                gg_SecOps_Adms=Users("SecOps_Adms")
                gg_SecOps_Adms \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("LabvelSecOpsAdms")
                mm_SecOps_Adms=[User("w.alejovl+secops\n-labs@gmail.com"),] 
                gg_SecOps_Adms \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_SecOps_Adms 


        with Cluster('Group: AWSControlTowerAdmins'):

                gg_AWSControlTowerAdmins=Users("AWSControlTowerA\ndmins")
                gg_AWSControlTowerAdmins \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSAdministrator\nAccess")
                mm_AWSControlTowerAdmins=[User("velez94@protonma\nil.com"),] 
                gg_AWSControlTowerAdmins \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSControlTowerAdmins 


        with Cluster('Group: AWSSecurityAuditors'):

                gg_AWSSecurityAuditors=Users("AWSSecurityAudit\nors")
                gg_AWSSecurityAuditors \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSReadOnlyAccess")
                mm_AWSSecurityAuditors=[] 
                gg_AWSSecurityAuditors \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditors 


    with Cluster('Account: DevSecOps'):

        with Cluster('Group: AWSSecurityAuditPowerUsers'):

                gg_AWSSecurityAuditPowerUsers=Users("AWSSecurityAudit\nPowerUsers")
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSPowerUserAcce\nss")
                mm_AWSSecurityAuditPowerUsers=[] 
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditPowerUsers 


        with Cluster('Group: DevSecOps_Admins'):

                gg_DevSecOps_Admins=Users("DevSecOps_Admins")
                gg_DevSecOps_Admins \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("LabvelDevSecOpsU\nsers")
                mm_DevSecOps_Admins=[User("DevSecOpsAdm"),] 
                gg_DevSecOps_Admins \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_DevSecOps_Admins 


        with Cluster('Group: DevSecOps_Admins'):

                gg_DevSecOps_Admins=Users("DevSecOps_Admins")
                gg_DevSecOps_Admins \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSAdministrator\nAccess")
                mm_DevSecOps_Admins=[User("DevSecOpsAdm"),] 
                gg_DevSecOps_Admins \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_DevSecOps_Admins 


        with Cluster('User: w.alejovl+devsecops-labs@gmail.com'):

                uu_walejovldevsecopslabsgmailcom=User("w.alejovl+devsec\nops-labs@gmail.com")
                uu_walejovldevsecopslabsgmailcom \
                        - Edge(color="brown", style="dotted") \
                        - IAMPermissions("AWSAdministrator\nAccess")

        with Cluster('Group: DevSecOps_Admins'):

                gg_DevSecOps_Admins=Users("DevSecOps_Admins")
                gg_DevSecOps_Admins \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("LabvelDevSecOpsRW")
                mm_DevSecOps_Admins=[User("DevSecOpsAdm"),] 
                gg_DevSecOps_Admins \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_DevSecOps_Admins 


        with Cluster('Group: AWSSecurityAuditors'):

                gg_AWSSecurityAuditors=Users("AWSSecurityAudit\nors")
                gg_AWSSecurityAuditors \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSReadOnlyAccess")
                mm_AWSSecurityAuditors=[] 
                gg_AWSSecurityAuditors \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditors 


    with Cluster('Account: Ops'):

        with Cluster('Group: AWSSecurityAuditPowerUsers'):

                gg_AWSSecurityAuditPowerUsers=Users("AWSSecurityAudit\nPowerUsers")
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSPowerUserAcce\nss")
                mm_AWSSecurityAuditPowerUsers=[] 
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditPowerUsers 


        with Cluster('User: w.alejovl+ct-labs@gmail.com'):

                uu_walejovlctlabsgmailcom=User("w.alejovl+ct-lab\ns@gmail.com")
                uu_walejovlctlabsgmailcom \
                        - Edge(color="brown", style="dotted") \
                        - IAMPermissions("AWSAdministrator\nAccess")

        with Cluster('Group: AWSSecurityAuditors'):

                gg_AWSSecurityAuditors=Users("AWSSecurityAudit\nors")
                gg_AWSSecurityAuditors \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSReadOnlyAccess")
                mm_AWSSecurityAuditors=[] 
                gg_AWSSecurityAuditors \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditors 


    with Cluster('Account: SharedServices'):

        with Cluster('Group: AWSSecurityAuditPowerUsers'):

                gg_AWSSecurityAuditPowerUsers=Users("AWSSecurityAudit\nPowerUsers")
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSPowerUserAcce\nss")
                mm_AWSSecurityAuditPowerUsers=[] 
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditPowerUsers 


        with Cluster('Group: DevSecOps_Admins'):

                gg_DevSecOps_Admins=Users("DevSecOps_Admins")
                gg_DevSecOps_Admins \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSAdministrator\nAccess")
                mm_DevSecOps_Admins=[User("DevSecOpsAdm"),] 
                gg_DevSecOps_Admins \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_DevSecOps_Admins 


        with Cluster('User: w.alejovl+shared-labs@gmail.com'):

                uu_walejovlsharedlabsgmailcom=User("w.alejovl+shared\n-labs@gmail.com")
                uu_walejovlsharedlabsgmailcom \
                        - Edge(color="brown", style="dotted") \
                        - IAMPermissions("AWSAdministrator\nAccess")

        with Cluster('Group: AWSSecurityAuditors'):

                gg_AWSSecurityAuditors=Users("AWSSecurityAudit\nors")
                gg_AWSSecurityAuditors \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSReadOnlyAccess")
                mm_AWSSecurityAuditors=[] 
                gg_AWSSecurityAuditors \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditors 


    with Cluster('Account: LabVel'):

        with Cluster('Group: AWSSecurityAuditPowerUsers'):

                gg_AWSSecurityAuditPowerUsers=Users("AWSSecurityAudit\nPowerUsers")
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSPowerUserAcce\nss")
                mm_AWSSecurityAuditPowerUsers=[] 
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditPowerUsers 


        with Cluster('Group: AWSAccountFactory'):

                gg_AWSAccountFactory=Users("AWSAccountFactory")
                gg_AWSAccountFactory \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSServiceCatalo\ngEndUserAccess")
                mm_AWSAccountFactory=[User("velez94@protonma\nil.com"),] 
                gg_AWSAccountFactory \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSAccountFactory 


        with Cluster('Group: AWSControlTowerAdmins'):

                gg_AWSControlTowerAdmins=Users("AWSControlTowerA\ndmins")
                gg_AWSControlTowerAdmins \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSAdministrator\nAccess")
                mm_AWSControlTowerAdmins=[User("velez94@protonma\nil.com"),] 
                gg_AWSControlTowerAdmins \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSControlTowerAdmins 


        with Cluster('Group: AWSSecurityAuditors'):

                gg_AWSSecurityAuditors=Users("AWSSecurityAudit\nors")
                gg_AWSSecurityAuditors \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSReadOnlyAccess")
                mm_AWSSecurityAuditors=[] 
                gg_AWSSecurityAuditors \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditors 


    with Cluster('Account: OrganizationManager'):

        with Cluster('Group: AWSSecurityAuditPowerUsers'):

                gg_AWSSecurityAuditPowerUsers=Users("AWSSecurityAudit\nPowerUsers")
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSPowerUserAcce\nss")
                mm_AWSSecurityAuditPowerUsers=[] 
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditPowerUsers 


        with Cluster('Group: SecOps_Adms'):

                gg_SecOps_Adms=Users("SecOps_Adms")
                gg_SecOps_Adms \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("LabvelSecOpsAdms")
                mm_SecOps_Adms=[User("w.alejovl+secops\n-labs@gmail.com"),] 
                gg_SecOps_Adms \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_SecOps_Adms 


        with Cluster('User: w.alejovl+orgman-labs@gmail.com'):

                uu_walejovlorgmanlabsgmailcom=User("w.alejovl+orgman\n-labs@gmail.com")
                uu_walejovlorgmanlabsgmailcom \
                        - Edge(color="brown", style="dotted") \
                        - IAMPermissions("AWSAdministrator\nAccess")

        with Cluster('Group: AWSSecurityAuditors'):

                gg_AWSSecurityAuditors=Users("AWSSecurityAudit\nors")
                gg_AWSSecurityAuditors \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSReadOnlyAccess")
                mm_AWSSecurityAuditors=[] 
                gg_AWSSecurityAuditors \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditors 


    with Cluster('Account: Dev'):

        with Cluster('Group: AWSSecurityAuditPowerUsers'):

                gg_AWSSecurityAuditPowerUsers=Users("AWSSecurityAudit\nPowerUsers")
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSPowerUserAcce\nss")
                mm_AWSSecurityAuditPowerUsers=[] 
                gg_AWSSecurityAuditPowerUsers \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditPowerUsers 


        with Cluster('Group: DevSecOps_Admins'):

                gg_DevSecOps_Admins=Users("DevSecOps_Admins")
                gg_DevSecOps_Admins \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSAdministrator\nAccess")
                mm_DevSecOps_Admins=[User("DevSecOpsAdm"),] 
                gg_DevSecOps_Admins \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_DevSecOps_Admins 


        with Cluster('User: w.alejovl+dev-labs@gmail.com'):

                uu_walejovldevlabsgmailcom=User("w.alejovl+dev-la\nbs@gmail.com")
                uu_walejovldevlabsgmailcom \
                        - Edge(color="brown", style="dotted") \
                        - IAMPermissions("AWSAdministrator\nAccess")

        with Cluster('Group: AWSSecurityAuditors'):

                gg_AWSSecurityAuditors=Users("AWSSecurityAudit\nors")
                gg_AWSSecurityAuditors \
                        - Edge(color="brown", style="dotted", label="Permissions Set") \
                        - IAMPermissions("AWSReadOnlyAccess")
                mm_AWSSecurityAuditors=[] 
                gg_AWSSecurityAuditors \
                        - Edge(color="darkgreen", style="dotted", label="Member") \
                        - mm_AWSSecurityAuditors 




import boto3

import logging

profile = "labvel-master"
boto3.setup_default_session(profile_name=profile)


def list_groups_pag(identity_store_id,client=boto3.client('identitystore', region_name="us-east-2"), next_token: str = None):
    paginator = client.get_paginator('list_groups')
    response_iterator = paginator.paginate(
        IdentityStoreId=identity_store_id,
        PaginationConfig={
            'MaxItems': 1000,
            'PageSize': 4,
            'StartingToken': next_token
        }
    )
    response = response_iterator.build_full_result()
    logging.info(response_iterator.build_full_result())
    return response["Groups"]


def list_groups(identity_store_id, client=boto3.client('identitystore', region_name="us-east-2"), ):
    groups = client.list_groups(
        IdentityStoreId=identity_store_id,
        MaxResults = 2
    )

    logging.info(groups)
    l_groups = groups["Groups"]
    logging.info(len(groups["Groups"]))

    if len(groups["Groups"]) >= 2:
        logging.info("Paginating ...")
        ad_groups = list_groups_pag(identity_store_id=identity_store_id,client= client, next_token=groups["NextToken"])
        for ad in ad_groups:
            l_groups.append(ad)
        logging.info(f"You have {len(l_groups)} Groups")
    print(len(l_groups))
    return l_groups


list_groups(identity_store_id="d-9a672b3314")



"""Describe SSO."""
import logging

from boto3 import client


def list_instances(region: str):
    """
    List all instances in the region.

    :param region:
    :return:
    """
    sso_client = client("sso-admin", region_name=region)
    response = sso_client.list_instances()

    return response["Instances"]


# def list account assignments with pagination
def list_account_assignments_pag(
    instance_arn, account_id, permission_set_arn, region, next_token
):
    """
    List all account assignments.

    :param instance_arn:
    :param account_id:
    :param permission_set_arn:
    :param region:
    :return:

    """
    sso_client = client("sso-admin", region_name=region)
    paginator = sso_client.get_paginator("list_account_assignments")
    response_iterator = paginator.paginate(
        InstanceArn=instance_arn,
        AccountId=account_id,
        PermissionSetArn=permission_set_arn,
        PaginationConfig={
            "MaxItems": 1000,
            "PageSize": 20,
            "StartingToken": next_token,
        },
    )
    return response_iterator["AccountAssignments"]


def list_account_assignments(
    instance_arn, account_id, permission_set_arn, region, sso_client
):
    """
    List all account assignments.

    :param sso_client:
    :param instance_arn:
    :param account_id:
    :param permission_set_arn:
    :param region:
    :return:

    """
    response = sso_client.list_account_assignments(
        InstanceArn=instance_arn,
        AccountId=account_id,
        PermissionSetArn=permission_set_arn,
        MaxResults=50,
    )
    account_assignments = response["AccountAssignments"]
    if len(response["AccountAssignments"]) >= 50:
        logging.info("Paginating ...")
        response_iterator = list_account_assignments_pag(
            instance_arn,
            account_id,
            permission_set_arn,
            region,
            next_token=response["NextToken"],
        )
        for response in response_iterator:
            logging.debug(response)
            account_assignments.append(response["AccountAssignments"])
            logging.info(response["AccountAssignments"])
    return account_assignments


def list_permissions_set_pag(instance_arn, region, next_token):
    """
    List all permission set in a region.

    :param next_token:
    :param instance_arn:
    :param region:
    :return:
    """
    sso_client = client("sso-admin", region_name=region)
    paginator = sso_client.get_paginator("list_permission_sets")
    response_iterator = paginator.paginate(
        InstanceArn=instance_arn,
        PaginationConfig={
            "MaxItems": 1000,
            "PageSize": 20,
            "StartingToken": next_token,
        },
    )
    response_iterator = response_iterator.build_full_result()
    return response_iterator["PermissionSets"]


def list_permissions_set(instance_arn, region):
    """
    List all permission set in a region.

    :param instance_arn:
    :param region:
    :return:
    """
    sso_client = client(
        "sso-admin",
        region_name=region,
    )
    response = sso_client.list_permission_sets(InstanceArn=instance_arn, MaxResults=20)
    logging.debug(response)
    permissions_set = response["PermissionSets"]

    if len(response["PermissionSets"]) >= 20:
        logging.info("Paginating ...")
        response_iterator = list_permissions_set_pag(
            instance_arn, region, next_token=response["NextToken"]
        )
        for response in response_iterator:
            logging.debug(response)
            permissions_set.append(response)
            logging.info(response)

    return permissions_set


def list_permission_provisioned(account_id, instance_arn, region):
    """
    List permission provisioned.

    :param account_id:
    :param instance_arn:
    :param region:
    :return:
    """
    l_client = client("sso-admin", region_name=region)
    response = l_client.list_permission_sets_provisioned_to_account(
        InstanceArn=instance_arn,
        AccountId=account_id,
    )
    logging.debug(response)
    return response["PermissionSets"]


def extends_permissions_set(permissions_sets, store_arn, region):
    """
    List all permission set in a region.

    :param permissions_sets:
    :param store_arn:
    :param region:
    :return:
    """
    sso_client = client("sso-admin", region_name=region)
    l_permissions_set_arn_name = {}
    for p in permissions_sets:
        response = sso_client.describe_permission_set(
            InstanceArn=store_arn, PermissionSetArn=p
        )

        l_permissions_set_arn_name[p] = response["PermissionSet"]["Name"]

        logging.debug(response["PermissionSet"]["Name"])
    return l_permissions_set_arn_name




from diagrams import Diagram, Cluster

from diagrams.aws.management import Organizations, OrganizationsAccount, OrganizationsOrganizationalUnit

with Diagram("Organizations-State", show=False, direction="TB"):
    ou = OrganizationsOrganizationalUnit("OU")
    oa = OrganizationsAccount("Account")

    with Cluster('Organizations'):

        oo = Organizations('o-9tlhkjyoii\n029921763173\nr-w3ow')

        ou_Sandbox= OrganizationsOrganizationalUnit("ou-w3ow-1sumtdvp\nSandbox")

        oo>> ou_Sandbox

        ou_Security= OrganizationsOrganizationalUnit("ou-w3ow-oqvta8tc\nSecurity")

        oo>> ou_Security

        ou_Workloads= OrganizationsOrganizationalUnit("ou-w3ow-1lpmyfug\nWorkloads")

        oo>> ou_Workloads

        ou_Dev= OrganizationsOrganizationalUnit("ou-w3ow-k24p2opx\nDev")

        oo>> ou_Dev

        ou_DevSecOps= OrganizationsOrganizationalUnit("ou-w3ow-b334bby6\nDevSecOps")

        oo>> ou_DevSecOps

        ou_Core= OrganizationsOrganizationalUnit("ou-w3ow-93hiq3zr\nCore")

        oo>> ou_Core

        ou_PolicyStaging= OrganizationsOrganizationalUnit("ou-w3ow-18verpsm\nPolicy Staging")

        oo>> ou_PolicyStaging

        ou_Suspended= OrganizationsOrganizationalUnit("ou-w3ow-7vunsbkd\nSuspended")

        oo>> ou_Suspended

        ou_Shared= OrganizationsOrganizationalUnit("ou-w3ow-w7dzhzcz\nShared")

        oo>> ou_Shared

        ou_Infrastructure= OrganizationsOrganizationalUnit("ou-w3ow-9q06w8rz\nInfrastructure")

        oo>> ou_Infrastructure

        ou_BULab= OrganizationsOrganizationalUnit("ou-w3ow-qa633svy\nBU-Lab")

        ou_Workloads>> ou_BULab

        ou_Prod= OrganizationsOrganizationalUnit("ou-w3ow-4sdr4ejy\nProd")

        ou_BULab>> ou_Prod

        ou_SDLC= OrganizationsOrganizationalUnit("ou-w3ow-vop5vccd\nSDLC")

        ou_BULab>> ou_SDLC

        ou_Core>> OrganizationsAccount("884478634998\nLog archive")

        ou_Security>> OrganizationsAccount("835863553119\nSecOps")

        ou_Prod>> OrganizationsAccount("582441254763\nProd")

        ou_Core>> OrganizationsAccount("895882538541\nSecurityTooling")

        ou_DevSecOps>> OrganizationsAccount("105171185823\nDevSecOps")

        ou_Infrastructure>> OrganizationsAccount("994261317734\nOps")

        ou_Infrastructure>> OrganizationsAccount("155794986228\nSharedServices")

        oo >> OrganizationsAccount("904985504252\nClowPiloto")

        oo >> OrganizationsAccount("029921763173\nLabVel")

        ou_Security>> OrganizationsAccount("837696987585\nOrganizationMana\nger")

        oo >> OrganizationsAccount("571340586587\nDev")



import json
from rich.console import Console
from rich.columns import Columns
from rich.panel import Panel
from rich.progress import track


# load json file from path function
def load_json(path):
    """
    Load json file.

    :param path:
    :return:
    """
    with open(path, 'r') as f:
        return json.load(f)


def get_members(group):
    """
    Get members name.

    :param group:
    :return:
    """
    members_groups = {}
    for m in group:
        members_groups[group[m]["group_name"]] = []
        for mm in group[m]['members']:
            members_groups[group[m]["group_name"]].append(mm["MemberId"]["UserName"])

    return members_groups


def pretty_members(members):
    """
    Print pretty members.

    :param members:
    :return:
    """
    string = ""
    for m in members:
        string += f"{m}\n"

    return string


def create_console_view(file_path="diagrams/json/groups.json"):
    """
    Create tree.

    :param file_path:
    :return:
    """
    c = load_json(file_path)
    members = get_members(c)
    console = Console()
    c = [Panel(f"[b][green]{group}[/b]\n[yellow]{pretty_members(members[group])}", expand=True) for group in c]
    console.print(Columns(c))


# get account  assignments
def pretty_account_assignments(accounts: dict):
    """
    Print pretty members.

    :type accounts: object
    :return:
    """
    string = ""

    for a in accounts.keys():
        for c in accounts[a]:

            if c['PrincipalType'] == "GROUP" and "GroupName" in c.keys():
                string += f"GroupName: {c['GroupName']}\n"
            elif c['PrincipalType'] == "USER":
                string += f"UserName: {c['UserName']}\n"
            if "PermissionSetName" in c.keys():
                string += f"PermissionSetName: {c['PermissionSetName']}\n\n"

    return string


def create_account_assignments_view(file_path="diagrams/json/account_assignments.json"):
    """
    Create tree.

    :param file_path:
    :return:
    """
    assign = load_json(file_path)

    console = Console()
    c = [Panel(f"[b]{account}[/b]\n[blue]{pretty_account_assignments(assign)}", expand=True) for account in assign]
    console.print(Columns(c))

# pretty for organizations



"""Describe Organizations."""
import logging
import os

import emoji
from colorama import Fore

from ..dgms.graph_mapper import create_file, create_mapper
from ..dgms.graph_template import graph_template
from ..reports.save_results import save_results
from .describe_sso import client


def describe_organization(region):
    """
    Describe the organization.

    :param region: AWS Region
    :return:
    """
    print(f"{Fore.GREEN}❇️ Describe Organization {Fore.RESET}")
    org_client = client("organizations", region_name=region)
    organization = org_client.describe_organization()
    organization = organization["Organization"]
    return organization


def list_roots(region):
    """
    List the roots.

    :param region: AWS Region
    :return:
    """
    org_client = client("organizations", region_name=region)
    roots = org_client.list_roots()
    return roots["Roots"]


# def list organizational units pagination
def list_organizational_units_pag(parent_id, region, next_token=None):
    """
    List organizational Units with pagination.

    :param parent_id:
    :param region:
    :param next_token:
    :return:
    """
    org_client = client("organizations", region_name=region)
    paginator = org_client.get_paginator("list_organizational_units_for_parent")
    response_iterator = paginator.paginate(
        ParentId=parent_id,
        PaginationConfig={"MaxItems": 1000, "PageSize": 4, "StartingToken": next_token},
    )
    response_iterator = response_iterator.build_full_result()
    return response_iterator["OrganizationalUnits"]


def list_organizational_units(parent_id, region, org_units=None):
    """
    List Organizational units.

    :param parent_id:
    :param region:
    :param org_units:
    :return:
    """
    org_client = client("organizations", region_name=region)
    if org_units is None:
        org_units = []
    ous = org_client.list_organizational_units_for_parent(
        ParentId=parent_id,
        MaxResults=20,
    )
    ous = ous["OrganizationalUnits"]

    if len(ous) >= 20:
        logging.info("Paginating ...")
        add_ous = list_organizational_units_pag(
            parent_id, region, next_token=ous["NextToken"]
        )
        for ou in add_ous:
            ous.append(ou)
        logging.debug(add_ous)

    for o in ous:
        org_units.append(o)
    logging.debug(
        f"The parent Id is: {parent_id}",
    )
    logging.debug(ous)
    if len(ous) > 0:
        for ou in ous:
            logging.debug(ou)
            if "Id" in ou.keys():
                logging.debug(f"Search nested for: {ou['Name']}")
                ous_next = list_organizational_units(
                    ou["Id"], region=region, org_units=org_units
                )
                logging.debug(ous_next)
                if len(ous_next) > 0:
                    logging.debug("Find Netsted")

    return org_units


def list_parents(child_id, region):
    """
    List the parents of a child.

    :param child_id:
    :param region:
    :return:
    """
    org_client = client("organizations", region_name=region)
    response = org_client.list_parents(
        ChildId=child_id,
    )
    return response["Parents"]


def index_ous(list_ous, region):
    """
    Index the parents of a child.

    :param list_ous:
    :param region:
    :return list_ous:

    """
    org_client = client("organizations", region_name=region)
    for ou in list_ous:
        if "Id" in ou.keys() and len(ou) > 0:
            response = org_client.list_parents(
                ChildId=ou["Id"],
            )
            logging.debug(response["Parents"])
            ou["Parents"] = response["Parents"]
    return list_ous


def list_accounts_pag(region, next_token: str = None):
    """
    List accounts with pagination.

    :param region:
    :param next_token:
    :return:
    """
    org_client = client("organizations", region_name=region)
    paginator = org_client.get_paginator("list_accounts")
    response_iterator = paginator.paginate(
        PaginationConfig={"MaxItems": 1000, "PageSize": 20, "StartingToken": next_token}
    )
    response = response_iterator.build_full_result()
    logging.info(response_iterator.build_full_result())
    return response["Accounts"]


def list_accounts(region):
    """
    List accounts.

    :param region:
    :return:
    """
    org_client = client("organizations", region_name=region)
    accounts = org_client.list_accounts()
    logging.info(accounts)
    l_account = accounts["Accounts"]
    logging.info(len(accounts["Accounts"]))
    if len(accounts["Accounts"]) >= 20:
        logging.info("Paginating ...")
        ad_accounts = list_accounts_pag(region=region, next_token=accounts["NextToken"])
        for ad in ad_accounts:
            l_account.append(ad)
        logging.info(f"You Organizations have {len(l_account)} Accounts")

    return l_account


def index_accounts(list_account, region):
    """
    Index accounts.

    :param list_account:
    :param region:
    :return:
    """
    accounts = []
    org_client = client("organizations", region_name=region)
    for a in list_account:
        response = org_client.list_parents(
            ChildId=a["Id"],
        )

        accounts.append(
            {"account": a["Id"], "name": a["Name"], "parents": response["Parents"]}
        )
    return accounts


def graph_organizations(diagrams_path, region, auto):
    """
    Create organizations Graph.

    :param auto:
    :param diagrams_path:
    :param region:
    :return:
    """
    template_file = "graph_org.py"
    code_path = f"{diagrams_path}/code"
    json_path = f"{diagrams_path}/json"
    create_file(
        template_content=graph_template,
        file_name=template_file,
        directory_path=code_path,
    )

    organization = describe_organization(region=region)
    print(Fore.BLUE + emoji.emojize(":sparkle: Getting Organization Info" + Fore.RESET))
    logging.debug(organization)
    logging.debug("The Roots Info")
    roots = list_roots(region=region)
    logging.debug(roots)

    print(
        Fore.BLUE
        + emoji.emojize(":sparkle: Listing Organizational Units " + Fore.RESET)
    )
    logging.debug("The Organizational Units list ")
    ous = list_organizational_units(parent_id=roots[0]["Id"], region=region)
    logging.debug(ous)
    logging.debug("The Organizational Units list with parents info")
    i_ous = index_ous(ous, region=region)
    logging.debug(i_ous)

    print(
        Fore.BLUE
        + emoji.emojize(":sparkle: Getting the Account list info" + Fore.RESET)
    )
    l_accounts = list_accounts(region=region)
    logging.debug(l_accounts)
    logging.debug("The Account list with parents info")

    print(
        Fore.YELLOW
        + emoji.emojize(
            f":information:  There are {len(l_accounts)} Accounts in your organization"
            + Fore.RESET
        )
    )
    i_accounts = index_accounts(l_accounts, region=region)
    logging.debug(i_accounts)

    file_name = "organizations.json"
    save_results(results=i_accounts, filename=file_name, directory_path=json_path)

    create_mapper(
        template_file=f"{code_path}/{template_file}",
        org=organization,
        root_id=roots[0]["Id"],
        list_ous=ous,
        list_accounts=i_accounts,
    )
    if auto:
        print(f"{Fore.GREEN}❇️ Creating diagrams in {code_path}")
        command = os.system(f"cd {code_path} && python3 {template_file}")
        if command != 0:
            print(Fore.RED + "❌ Error creating diagrams")
            print(command)

    else:
        print(
            Fore.YELLOW
            + emoji.emojize(
                f":sparkles: Run -> python3 {code_path}/graph_org.py " + Fore.RESET
            )
        )



"""Create graphs."""
import argparse
import logging

import argcomplete
from boto3 import setup_default_session

from .aws.describe_identity_store import graph_identity_center
from .aws.describe_organization import graph_organizations
from .banner.banner import get_version
from .version import __version__



def main() -> int:
    """
    Crete architecture diagram from your current state.

    :return:
    """
    # Initialize parser
    parser = argparse.ArgumentParser()

    parser.add_argument(
        "-p", "--profile", help="AWS cli profile for AWS Apis", default=None
    )
    parser.add_argument(
        "-od",
        "--output_dir_path",
        help="Name of folder to save the diagrams python code files",
        default="diagrams",
    )
    parser.add_argument("-r", "--region", help="AWS region", default="us-east-2")
    parser.add_argument(
        "-o",
        "--graph_organization",
        help="Set if you want to create graph for your organization",
        action="store_true",
    )
    parser.add_argument(
        "-i",
        "--graph_identity",
        help="Set if you want to create graph for your IAM Center",
        action="store_true",
    )
    parser.add_argument(
        "-a",
        "--auto_create",
        help="Create Automatically diagrams",
        action="store_true",
        default=True,
    )
    parser.add_argument("-v", "--version", help="Show version", action="store_true")
    parser.add_argument("-d", "--debug", help="Debug Mode", action="store_true")

    # Read arguments from command line
    args = parser.parse_args()
    # Add autocomplete
    argcomplete.autocomplete(parser)
    logging.info(f"The arguments are {args}")
    if args.debug:
        logging.basicConfig(level=logging.DEBUG)

    diagrams_path = args.output_dir_path

    region = args.region

    if args.profile:
        profile = args.profile
        if profile is not None:
            setup_default_session(profile_name=profile)

        logging.info(f"Profile is: {profile}")

    if args.graph_organization:
        graph_organizations(
            diagrams_path=diagrams_path, region=region, auto=args.auto_create
        )

    if args.graph_identity:
        graph_identity_center(
            diagrams_path=diagrams_path, region=region, auto=args.auto_create
        )

    if args.version:
        get_version(version=__version__)

    return 0


if __name__ == "__main__":
    main()



"""Save results."""
import json
import logging
from pathlib import Path

from colorama import Fore


def save_results(results, filename, directory_path="."):
    """
    Save results to a file.

    :param directory_path:
    :param results:
    :param filename:

    :return: None. Saves results to a file.
    """
    if not Path.exists(Path(directory_path)):
        Path.mkdir(Path(directory_path))
        logging.debug(f"Directory {directory_path} created")
    with open(f"{directory_path}/{filename}", "w") as f:
        json.dump(results, fp=f, indent=4)
        print(
            f"{Fore.YELLOW}ℹ️  The accounts are stored in {directory_path}/{filename} {Fore.RESET}"
        )




from diagrams import Diagram, Cluster

from diagrams.aws.management import Organizations, OrganizationsAccount, OrganizationsOrganizationalUnit
from diagrams.aws.general import Users, User

with Diagram("SSO-State", show=False, direction="TB"):
    gg = Users("Group")
    uu= User("User")

    with Cluster('Groups'):

        gg_0= Users("AWSSecurityAudit\nPowerUsers")

        with Cluster("SecOps_Adms"):

                gg_1= [User("w.alejovl+secops\n-labs@gmail.com"),]

        gg_2= Users("AWSServiceCatalo\ngAdmins")

        with Cluster("AWSControlTowerAdmins"):

                gg_3= [User("velez94@protonma\nil.com"),]

        with Cluster("AWSAccountFactory"):

                gg_4= [User("velez94@protonma\nil.com"),]

        with Cluster("DevSecOps_Admins"):

                gg_5= [User("DevSecOpsAdm"),]

        gg_6= Users("AWSLogArchiveAdm\nins")

        gg_7= Users("AWSLogArchiveVie\nwers")

        gg_8= Users("AWSSecurityAudit\nors")



import logging
import re

f_accounts = {'Log archive': [{'AccountId': '884478634998',
                               'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-ab185f05acde5e90',
                               'PrincipalType': 'GROUP',
                               'PrincipalId': '9a672b3314-b858476a-2ef9-4018-90e7-29e5e4bc4388',
                               'GroupName': 'AWSSecurityAuditPowerUsers', 'PermissionSetName': 'AWSPowerUserAccess'},
                              {'AccountId': '884478634998',
                               'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-21058a9d1f62c7e2',
                               'PrincipalType': 'GROUP',
                               'PrincipalId': '9a672b3314-43117aac-887b-48ee-af49-b6b6cd059199',
                               'GroupName': 'AWSControlTowerAdmins', 'PermissionSetName': 'AWSAdministratorAccess'},
                              {'AccountId': '884478634998',
                               'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-21058a9d1f62c7e2',
                               'PrincipalType': 'GROUP',
                               'PrincipalId': '9a672b3314-c481fbee-8062-432a-8b87-eeaa36b763a8',
                               'GroupName': 'AWSLogArchiveAdmins', 'PermissionSetName': 'AWSAdministratorAccess'},
                              {'AccountId': '884478634998',
                               'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-cf27b0efdc941a09',
                               'PrincipalType': 'GROUP',
                               'PrincipalId': '9a672b3314-f46f413e-44d7-4d3d-918b-f86721413097',
                               'GroupName': 'AWSSecurityAuditors', 'PermissionSetName': 'AWSReadOnlyAccess'},
                              {'AccountId': '884478634998',
                               'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-cf27b0efdc941a09',
                               'PrincipalType': 'GROUP',
                               'PrincipalId': '9a672b3314-ff479c57-03cb-440e-8902-be8ea9d7d25b',
                               'GroupName': 'AWSLogArchiveViewers', 'PermissionSetName': 'AWSReadOnlyAccess'}],
              'SecOps': [{'AccountId': '835863553119',
                          'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-ab185f05acde5e90',
                          'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-b858476a-2ef9-4018-90e7-29e5e4bc4388',
                          'GroupName': 'AWSSecurityAuditPowerUsers', 'PermissionSetName': 'AWSPowerUserAccess'},
                         {'AccountId': '835863553119',
                          'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-21058a9d1f62c7e2',
                          'PrincipalType': 'USER', 'PrincipalId': 'c1eb9540-e0f1-702f-c53f-405c01d9f991',
                          'UserName': 'w.alejovl+secops-labs@gmail.com', 'PermissionSetName': 'AWSAdministratorAccess'},
                         {'AccountId': '835863553119',
                          'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-cf27b0efdc941a09',
                          'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-f46f413e-44d7-4d3d-918b-f86721413097',
                          'GroupName': 'AWSSecurityAuditors', 'PermissionSetName': 'AWSReadOnlyAccess'},
                         {'AccountId': '835863553119',
                          'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-c6046bbbf15aaafc',
                          'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-43117aac-887b-48ee-af49-b6b6cd059199',
                          'GroupName': 'AWSControlTowerAdmins', 'PermissionSetName': 'AWSOrganizationsFullAccess'}],
              'Prod': [{'AccountId': '582441254763',
                        'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-ab185f05acde5e90',
                        'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-b858476a-2ef9-4018-90e7-29e5e4bc4388',
                        'GroupName': 'AWSSecurityAuditPowerUsers', 'PermissionSetName': 'AWSPowerUserAccess'},
                       {'AccountId': '582441254763',
                        'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-21058a9d1f62c7e2',
                        'PrincipalType': 'GROUP', 'PrincipalId': '318bc590-a071-70f5-63f6-ab21233e4e33',
                        'GroupName': 'DevSecOps_Admins', 'PermissionSetName': 'AWSAdministratorAccess'},
                       {'AccountId': '582441254763',
                        'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-21058a9d1f62c7e2',
                        'PrincipalType': 'USER', 'PrincipalId': '31eb65d0-f051-70a8-3786-9be14e01dc0e',
                        'UserName': 'w.alejovl+prod-labs@gmail.com', 'PermissionSetName': 'AWSAdministratorAccess'},
                       {'AccountId': '582441254763',
                        'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-cf27b0efdc941a09',
                        'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-f46f413e-44d7-4d3d-918b-f86721413097',
                        'GroupName': 'AWSSecurityAuditors', 'PermissionSetName': 'AWSReadOnlyAccess'},
                       {'AccountId': '582441254763',
                        'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-c6046bbbf15aaafc',
                        'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-43117aac-887b-48ee-af49-b6b6cd059199',
                        'GroupName': 'AWSControlTowerAdmins', 'PermissionSetName': 'AWSOrganizationsFullAccess'}],
              'SecurityTooling': [{'AccountId': '895882538541',
                                   'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-ab185f05acde5e90',
                                   'PrincipalType': 'GROUP',
                                   'PrincipalId': '9a672b3314-b858476a-2ef9-4018-90e7-29e5e4bc4388',
                                   'GroupName': 'AWSSecurityAuditPowerUsers',
                                   'PermissionSetName': 'AWSPowerUserAccess'}, {'AccountId': '895882538541',
                                                                                'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-83071c7372606026',
                                                                                'PrincipalType': 'GROUP',
                                                                                'PrincipalId': '913ba5c0-f071-70e1-05c5-a927586dc132',
                                                                                'GroupName': 'SecOps_Adms',
                                                                                'PermissionSetName': 'LabvelSecOpsAdms'},
                                  {'AccountId': '895882538541',
                                   'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-21058a9d1f62c7e2',
                                   'PrincipalType': 'GROUP',
                                   'PrincipalId': '9a672b3314-43117aac-887b-48ee-af49-b6b6cd059199',
                                   'GroupName': 'AWSControlTowerAdmins', 'PermissionSetName': 'AWSAdministratorAccess'},
                                  {'AccountId': '895882538541',
                                   'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-21058a9d1f62c7e2',
                                   'PrincipalType': 'GROUP',
                                   'PrincipalId': '9a672b3314-faf36c54-a70c-4db6-aefc-e5ac006ad5a1',
                                   'GroupName': 'AWSAuditAccountAdmins', 'PermissionSetName': 'AWSAdministratorAccess'},
                                  {'AccountId': '895882538541',
                                   'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-cf27b0efdc941a09',
                                   'PrincipalType': 'GROUP',
                                   'PrincipalId': '9a672b3314-f46f413e-44d7-4d3d-918b-f86721413097',
                                   'GroupName': 'AWSSecurityAuditors', 'PermissionSetName': 'AWSReadOnlyAccess'}],
              'DevSecOps': [{'AccountId': '105171185823',
                             'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-ab185f05acde5e90',
                             'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-b858476a-2ef9-4018-90e7-29e5e4bc4388',
                             'GroupName': 'AWSSecurityAuditPowerUsers', 'PermissionSetName': 'AWSPowerUserAccess'},
                            {'AccountId': '105171185823',
                             'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-002383fcbd1b90d7',
                             'PrincipalType': 'GROUP', 'PrincipalId': '318bc590-a071-70f5-63f6-ab21233e4e33',
                             'GroupName': 'DevSecOps_Admins', 'PermissionSetName': 'LabvelDevSecOpsUsers'},
                            {'AccountId': '105171185823',
                             'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-21058a9d1f62c7e2',
                             'PrincipalType': 'GROUP', 'PrincipalId': '318bc590-a071-70f5-63f6-ab21233e4e33',
                             'GroupName': 'DevSecOps_Admins', 'PermissionSetName': 'AWSAdministratorAccess'},
                            {'AccountId': '105171185823',
                             'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-21058a9d1f62c7e2',
                             'PrincipalType': 'USER', 'PrincipalId': '81bb65b0-40f1-7082-2b16-83138563c37b',
                             'UserName': 'w.alejovl+devsecops-labs@gmail.com',
                             'PermissionSetName': 'AWSAdministratorAccess'}, {'AccountId': '105171185823',
                                                                              'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-e370384c6f6d9b8a',
                                                                              'PrincipalType': 'GROUP',
                                                                              'PrincipalId': '318bc590-a071-70f5-63f6-ab21233e4e33',
                                                                              'GroupName': 'DevSecOps_Admins',
                                                                              'PermissionSetName': 'LabvelDevSecOpsRW'},
                            {'AccountId': '105171185823',
                             'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-cf27b0efdc941a09',
                             'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-f46f413e-44d7-4d3d-918b-f86721413097',
                             'GroupName': 'AWSSecurityAuditors', 'PermissionSetName': 'AWSReadOnlyAccess'},
                            {'AccountId': '105171185823',
                             'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-c6046bbbf15aaafc',
                             'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-43117aac-887b-48ee-af49-b6b6cd059199',
                             'GroupName': 'AWSControlTowerAdmins', 'PermissionSetName': 'AWSOrganizationsFullAccess'}],
              'Ops': [{'AccountId': '994261317734',
                       'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-ab185f05acde5e90',
                       'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-b858476a-2ef9-4018-90e7-29e5e4bc4388',
                       'GroupName': 'AWSSecurityAuditPowerUsers', 'PermissionSetName': 'AWSPowerUserAccess'},
                      {'AccountId': '994261317734',
                       'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-21058a9d1f62c7e2',
                       'PrincipalType': 'USER', 'PrincipalId': '9a672b3314-f7e9ac54-7407-4428-8562-f8e3afebbe76',
                       'UserName': 'w.alejovl+ct-labs@gmail.com', 'PermissionSetName': 'AWSAdministratorAccess'},
                      {'AccountId': '994261317734',
                       'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-cf27b0efdc941a09',
                       'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-f46f413e-44d7-4d3d-918b-f86721413097',
                       'GroupName': 'AWSSecurityAuditors', 'PermissionSetName': 'AWSReadOnlyAccess'},
                      {'AccountId': '994261317734',
                       'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-c6046bbbf15aaafc',
                       'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-43117aac-887b-48ee-af49-b6b6cd059199',
                       'GroupName': 'AWSControlTowerAdmins', 'PermissionSetName': 'AWSOrganizationsFullAccess'}],
              'SharedServices': [{'AccountId': '155794986228',
                                  'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-ab185f05acde5e90',
                                  'PrincipalType': 'GROUP',
                                  'PrincipalId': '9a672b3314-b858476a-2ef9-4018-90e7-29e5e4bc4388',
                                  'GroupName': 'AWSSecurityAuditPowerUsers', 'PermissionSetName': 'AWSPowerUserAccess'},
                                 {'AccountId': '155794986228',
                                  'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-21058a9d1f62c7e2',
                                  'PrincipalType': 'GROUP', 'PrincipalId': '318bc590-a071-70f5-63f6-ab21233e4e33',
                                  'GroupName': 'DevSecOps_Admins', 'PermissionSetName': 'AWSAdministratorAccess'},
                                 {'AccountId': '155794986228',
                                  'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-21058a9d1f62c7e2',
                                  'PrincipalType': 'USER', 'PrincipalId': '417b65f0-2021-707d-6483-08a1fa4997c6',
                                  'UserName': 'w.alejovl+shared-labs@gmail.com',
                                  'PermissionSetName': 'AWSAdministratorAccess'}, {'AccountId': '155794986228',
                                                                                   'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-cf27b0efdc941a09',
                                                                                   'PrincipalType': 'GROUP',
                                                                                   'PrincipalId': '9a672b3314-f46f413e-44d7-4d3d-918b-f86721413097',
                                                                                   'GroupName': 'AWSSecurityAuditors',
                                                                                   'PermissionSetName': 'AWSReadOnlyAccess'},
                                 {'AccountId': '155794986228',
                                  'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-c6046bbbf15aaafc',
                                  'PrincipalType': 'GROUP',
                                  'PrincipalId': '9a672b3314-43117aac-887b-48ee-af49-b6b6cd059199',
                                  'GroupName': 'AWSControlTowerAdmins',
                                  'PermissionSetName': 'AWSOrganizationsFullAccess'}], 'LabVel': [
        {'AccountId': '029921763173',
         'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-ab185f05acde5e90',
         'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-b858476a-2ef9-4018-90e7-29e5e4bc4388',
         'GroupName': 'AWSSecurityAuditPowerUsers', 'PermissionSetName': 'AWSPowerUserAccess'},
        {'AccountId': '029921763173',
         'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-7cc34a5a03379f6f',
         'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-f8065505-3174-4d46-a1b4-f134fd0ca2fc',
         'GroupName': 'AWSAccountFactory', 'PermissionSetName': 'AWSServiceCatalogEndUserAccess'},
        {'AccountId': '029921763173',
         'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-21058a9d1f62c7e2',
         'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-43117aac-887b-48ee-af49-b6b6cd059199',
         'GroupName': 'AWSControlTowerAdmins', 'PermissionSetName': 'AWSAdministratorAccess'},
        {'AccountId': '029921763173',
         'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-cf27b0efdc941a09',
         'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-f46f413e-44d7-4d3d-918b-f86721413097',
         'GroupName': 'AWSSecurityAuditors', 'PermissionSetName': 'AWSReadOnlyAccess'}, {'AccountId': '029921763173',
                                                                                         'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-83e7c23c8b2df8b3',
                                                                                         'PrincipalType': 'GROUP',
                                                                                         'PrincipalId': '9a672b3314-7f743f07-169a-4172-bdbc-561e7908e463',
                                                                                         'GroupName': 'AWSServiceCatalogAdmins',
                                                                                         'PermissionSetName': 'AWSServiceCatalogAdminFullAccess'}],
              'OrganizationManager': [{'AccountId': '837696987585',
                                       'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-ab185f05acde5e90',
                                       'PrincipalType': 'GROUP',
                                       'PrincipalId': '9a672b3314-b858476a-2ef9-4018-90e7-29e5e4bc4388',
                                       'GroupName': 'AWSSecurityAuditPowerUsers',
                                       'PermissionSetName': 'AWSPowerUserAccess'}, {'AccountId': '837696987585',
                                                                                    'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-83071c7372606026',
                                                                                    'PrincipalType': 'GROUP',
                                                                                    'PrincipalId': '913ba5c0-f071-70e1-05c5-a927586dc132',
                                                                                    'GroupName': 'SecOps_Adms',
                                                                                    'PermissionSetName': 'LabvelSecOpsAdms'},
                                      {'AccountId': '837696987585',
                                       'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-21058a9d1f62c7e2',
                                       'PrincipalType': 'USER', 'PrincipalId': 'c19b15f0-4001-7011-b67d-3e211db16680',
                                       'UserName': 'w.alejovl+orgman-labs@gmail.com',
                                       'PermissionSetName': 'AWSAdministratorAccess'}, {'AccountId': '837696987585',
                                                                                        'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-cf27b0efdc941a09',
                                                                                        'PrincipalType': 'GROUP',
                                                                                        'PrincipalId': '9a672b3314-f46f413e-44d7-4d3d-918b-f86721413097',
                                                                                        'GroupName': 'AWSSecurityAuditors',
                                                                                        'PermissionSetName': 'AWSReadOnlyAccess'},
                                      {'AccountId': '837696987585',
                                       'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-c6046bbbf15aaafc',
                                       'PrincipalType': 'GROUP',
                                       'PrincipalId': '9a672b3314-43117aac-887b-48ee-af49-b6b6cd059199',
                                       'GroupName': 'AWSControlTowerAdmins',
                                       'PermissionSetName': 'AWSOrganizationsFullAccess'}], 'Dev': [
        {'AccountId': '571340586587',
         'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-ab185f05acde5e90',
         'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-b858476a-2ef9-4018-90e7-29e5e4bc4388',
         'GroupName': 'AWSSecurityAuditPowerUsers', 'PermissionSetName': 'AWSPowerUserAccess'},
        {'AccountId': '571340586587',
         'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-21058a9d1f62c7e2',
         'PrincipalType': 'GROUP', 'PrincipalId': '318bc590-a071-70f5-63f6-ab21233e4e33',
         'GroupName': 'DevSecOps_Admins', 'PermissionSetName': 'AWSAdministratorAccess'}, {'AccountId': '571340586587',
                                                                                           'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-21058a9d1f62c7e2',
                                                                                           'PrincipalType': 'USER',
                                                                                           'PrincipalId': '41fbf590-8001-70d1-334c-2bbb4adb9989',
                                                                                           'UserName': 'w.alejovl+dev-labs@gmail.com',
                                                                                           'PermissionSetName': 'AWSAdministratorAccess'},
        {'AccountId': '571340586587',
         'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-cf27b0efdc941a09',
         'PrincipalType': 'GROUP', 'PrincipalId': '9a672b3314-f46f413e-44d7-4d3d-918b-f86721413097',
         'GroupName': 'AWSSecurityAuditors', 'PermissionSetName': 'AWSReadOnlyAccess'}, {'AccountId': '571340586587',
                                                                                         'PermissionSetArn': 'arn:aws:sso:::permissionSet/ssoins-66845289d6823727/ps-c6046bbbf15aaafc',
                                                                                         'PrincipalType': 'GROUP',
                                                                                         'PrincipalId': '9a672b3314-43117aac-887b-48ee-af49-b6b6cd059199',
                                                                                         'GroupName': 'AWSControlTowerAdmins',
                                                                                         'PermissionSetName': 'AWSOrganizationsFullAccess'}]}
c_users_and_groups = [
    {'group_id': '9a672b3314-f46f413e-44d7-4d3d-918b-f86721413097', 'group_name': 'AWSSecurityAuditors', 'members': []},
    {'group_id': '9a672b3314-c481fbee-8062-432a-8b87-eeaa36b763a8', 'group_name': 'AWSLogArchiveAdmins', 'members': []},
    {'group_id': '318bc590-a071-70f5-63f6-ab21233e4e33', 'group_name': 'DevSecOps_Admins', 'members': [
        {'IdentityStoreId': 'd-9a672b3314', 'MembershipId': '51bbe5a0-7001-7010-d7c0-46f5044d014e',
         'GroupId': '318bc590-a071-70f5-63f6-ab21233e4e33',
         'MemberId': {'UserId': '010be510-1061-70df-8274-96526bc47eb7', 'UserName': 'DevSecOpsAdm'}}]},
    {'group_id': '9a672b3314-ff479c57-03cb-440e-8902-be8ea9d7d25b', 'group_name': 'AWSLogArchiveViewers',
     'members': []},
    {'group_id': '9a672b3314-b858476a-2ef9-4018-90e7-29e5e4bc4388', 'group_name': 'AWSSecurityAuditPowerUsers',
     'members': []},
    {'group_id': '9a672b3314-faf36c54-a70c-4db6-aefc-e5ac006ad5a1', 'group_name': 'AWSAuditAccountAdmins',
     'members': []},
    {'group_id': '9a672b3314-f8065505-3174-4d46-a1b4-f134fd0ca2fc', 'group_name': 'AWSAccountFactory', 'members': [
        {'IdentityStoreId': 'd-9a672b3314', 'MembershipId': 'e18bb590-9031-70a0-5469-42c9799e8a6b',
         'GroupId': '9a672b3314-f8065505-3174-4d46-a1b4-f134fd0ca2fc',
         'MemberId': {'UserId': '9a672b3314-bd21c8b3-1aa0-4922-9374-92321b4979bf',
                      'UserName': 'velez94@protonmail.com'}}]},
    {'group_id': '9a672b3314-7f743f07-169a-4172-bdbc-561e7908e463', 'group_name': 'AWSServiceCatalogAdmins',
     'members': []},
    {'group_id': '9a672b3314-43117aac-887b-48ee-af49-b6b6cd059199', 'group_name': 'AWSControlTowerAdmins', 'members': [
        {'IdentityStoreId': 'd-9a672b3314', 'MembershipId': 'e14b3500-3051-70b2-25b7-d5729d383061',
         'GroupId': '9a672b3314-43117aac-887b-48ee-af49-b6b6cd059199',
         'MemberId': {'UserId': '9a672b3314-bd21c8b3-1aa0-4922-9374-92321b4979bf',
                      'UserName': 'velez94@protonmail.com'}}]}]


def l_groups_to_d_groups(l_groups: list = None):
    """

    :param l_groups:
    :return:
    """
    names = []
    for a in l_groups:
        names.append(a['group_name'])
    print(names)

    d_user_groups = dict(zip(names, c_users_and_groups))
    print(d_user_groups)

    return d_user_groups


def get_group_members_with_account(accounts: dict = None, g_dict: dict = None):
    for key, value in accounts.items():
        if len(value) > 0:
            for m in value:
                if "GroupName" in m.keys():

                    if m['GroupName'] in g_dict.keys():
                        print(
                            f"FindGroup  {g_dict[m['GroupName']]} with members {g_dict[m['GroupName']]['members']} in {key}")


d_users_group = l_groups_to_d_groups(c_users_and_groups)

get_group_members_with_account(accounts=f_accounts, g_dict=d_users_group)
print(d_users_group[ 'DevSecOps_Admins']['members'])